@@ -5,6 +5,7 @@
 from abc import ABCMeta, abstractmethod
 
 import numpy as np
+from numbers import Integral
 
 from scipy.linalg import norm
 from scipy.sparse import dia_matrix, issparse
@@ -13,10 +14,12 @@ from scipy.sparse.linalg import eigsh, svds
 from . import KMeans, MiniBatchKMeans
 from ..base import BaseEstimator, BiclusterMixin
 from ..utils import check_random_state
+from ..utils import check_scalar
 
 from ..utils.extmath import make_nonnegative, randomized_svd, safe_sparse_dot
 
 from ..utils.validation import assert_all_finite
+from ..utils._param_validation import Interval, StrOptions
 
 
 __all__ = ["SpectralCoclustering", "SpectralBiclustering"]
@@ -83,6 +86,15 @@ def _log_normalize(X):
 class BaseSpectral(BiclusterMixin, BaseEstimator, metaclass=ABCMeta):
     """Base class for spectral biclustering."""
 
+    _parameter_constraints: dict = {
+        "svd_method": [StrOptions({"randomized", "arpack"})],
+        "n_svd_vecs": [Interval(Integral, 0, None, closed="left"), None],
+        "mini_batch": ["boolean"],
+        "init": [StrOptions({"k-means++", "random"}), np.ndarray],
+        "n_init": [Interval(Integral, 1, None, closed="left")],
+        "random_state": ["random_state"],
+    }
+
     @abstractmethod
     def __init__(
         self,
@@ -102,14 +114,9 @@ class BaseSpectral(BiclusterMixin, BaseEstimator, metaclass=ABCMeta):
         self.n_init = n_init
         self.random_state = random_state
 
-    def _check_parameters(self):
-        legal_svd_methods = ("randomized", "arpack")
-        if self.svd_method not in legal_svd_methods:
-            raise ValueError(
-                "Unknown SVD method: '{0}'. svd_method must be one of {1}.".format(
-                    self.svd_method, legal_svd_methods
-                )
-            )
+    @abstractmethod
+    def _check_parameters(self, n_samples):
+        """Validate parameters depending on the input data."""
 
     def fit(self, X, y=None):
         """Create a biclustering for X.
@@ -127,8 +134,10 @@ class BaseSpectral(BiclusterMixin, BaseEstimator, metaclass=ABCMeta):
         self : object
             SpectralBiclustering instance.
         """
+        self._validate_params()
+
         X = self._validate_data(X, accept_sparse="csr", dtype=np.float64)
-        self._check_parameters()
+        self._check_parameters(X.shape[0])
         self._fit(X)
         return self
 
@@ -240,7 +249,7 @@ class SpectralCoclustering(BaseSpectral):
         Whether to use mini-batch k-means, which is faster but may get
         different results.
 
-    init : {'k-means++', 'random', or ndarray of shape \
+    init : {'k-means++', 'random'}, or ndarray of shape \
             (n_clusters, n_features), default='k-means++'
         Method for initialization of k-means algorithm; defaults to
         'k-means++'.
@@ -294,9 +303,9 @@ class SpectralCoclustering(BaseSpectral):
 
     References
     ----------
-    * Dhillon, Inderjit S, 2001. `Co-clustering documents and words using
-      bipartite spectral graph partitioning
-      <http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.140.3011>`__.
+    * :doi:`Dhillon, Inderjit S, 2001. Co-clustering documents and words using
+      bipartite spectral graph partitioning.
+      <10.1145/502512.502550>`
 
     Examples
     --------
@@ -313,6 +322,11 @@ class SpectralCoclustering(BaseSpectral):
     SpectralCoclustering(n_clusters=2, random_state=0)
     """
 
+    _parameter_constraints: dict = {
+        **BaseSpectral._parameter_constraints,
+        "n_clusters": [Interval(Integral, 1, None, closed="left")],
+    }
+
     def __init__(
         self,
         n_clusters=3,
@@ -328,6 +342,13 @@ class SpectralCoclustering(BaseSpectral):
             n_clusters, svd_method, n_svd_vecs, mini_batch, init, n_init, random_state
         )
 
+    def _check_parameters(self, n_samples):
+        if self.n_clusters > n_samples:
+            raise ValueError(
+                f"n_clusters should be <= n_samples={n_samples}. Got"
+                f" {self.n_clusters} instead."
+            )
+
     def _fit(self, X):
         normalized_data, row_diag, col_diag = _scale_normalize(X)
         n_sv = 1 + int(np.ceil(np.log2(self.n_clusters)))
@@ -372,7 +393,7 @@ class SpectralBiclustering(BaseSpectral):
         default is 'bistochastic'.
 
         .. warning::
-           if `method='log'`, the data must be sparse.
+           if `method='log'`, the data must not be sparse.
 
     n_components : int, default=6
         Number of singular vectors to check.
@@ -398,7 +419,7 @@ class SpectralBiclustering(BaseSpectral):
         Whether to use mini-batch k-means, which is faster but may get
         different results.
 
-    init : {'k-means++', 'random'} or ndarray of (n_clusters, n_features), \
+    init : {'k-means++', 'random'} or ndarray of shape (n_clusters, n_features), \
             default='k-means++'
         Method for initialization of k-means algorithm; defaults to
         'k-means++'.
@@ -452,9 +473,9 @@ class SpectralBiclustering(BaseSpectral):
     References
     ----------
 
-    * Kluger, Yuval, et. al., 2003. `Spectral biclustering of microarray
-      data: coclustering genes and conditions
-      <http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.135.1608>`__.
+    * :doi:`Kluger, Yuval, et. al., 2003. Spectral biclustering of microarray
+      data: coclustering genes and conditions.
+      <10.1101/gr.648603>`
 
     Examples
     --------
@@ -471,6 +492,14 @@ class SpectralBiclustering(BaseSpectral):
     SpectralBiclustering(n_clusters=2, random_state=0)
     """
 
+    _parameter_constraints: dict = {
+        **BaseSpectral._parameter_constraints,
+        "n_clusters": [Interval(Integral, 1, None, closed="left"), tuple],
+        "method": [StrOptions({"bistochastic", "scale", "log"})],
+        "n_components": [Interval(Integral, 1, None, closed="left")],
+        "n_best": [Interval(Integral, 1, None, closed="left")],
+    }
+
     def __init__(
         self,
         n_clusters=3,
@@ -492,45 +521,43 @@ class SpectralBiclustering(BaseSpectral):
         self.n_components = n_components
         self.n_best = n_best
 
-    def _check_parameters(self):
-        super()._check_parameters()
-        legal_methods = ("bistochastic", "scale", "log")
-        if self.method not in legal_methods:
-            raise ValueError(
-                "Unknown method: '{0}'. method must be one of {1}.".format(
-                    self.method, legal_methods
+    def _check_parameters(self, n_samples):
+        if isinstance(self.n_clusters, Integral):
+            if self.n_clusters > n_samples:
+                raise ValueError(
+                    f"n_clusters should be <= n_samples={n_samples}. Got"
+                    f" {self.n_clusters} instead."
                 )
-            )
-        try:
-            int(self.n_clusters)
-        except TypeError:
+        else:  # tuple
             try:
-                r, c = self.n_clusters
-                int(r)
-                int(c)
+                n_row_clusters, n_column_clusters = self.n_clusters
+                check_scalar(
+                    n_row_clusters,
+                    "n_row_clusters",
+                    target_type=Integral,
+                    min_val=1,
+                    max_val=n_samples,
+                )
+                check_scalar(
+                    n_column_clusters,
+                    "n_column_clusters",
+                    target_type=Integral,
+                    min_val=1,
+                    max_val=n_samples,
+                )
             except (ValueError, TypeError) as e:
                 raise ValueError(
                     "Incorrect parameter n_clusters has value:"
-                    " {}. It should either be a single integer"
+                    f" {self.n_clusters}. It should either be a single integer"
                     " or an iterable with two integers:"
                     " (n_row_clusters, n_column_clusters)"
+                    " And the values are should be in the"
+                    " range: (1, n_samples)"
                 ) from e
-        if self.n_components < 1:
-            raise ValueError(
-                "Parameter n_components must be greater than 0,"
-                " but its value is {}".format(self.n_components)
-            )
-        if self.n_best < 1:
-            raise ValueError(
-                "Parameter n_best must be greater than 0, but its value is {}".format(
-                    self.n_best
-                )
-            )
+
         if self.n_best > self.n_components:
             raise ValueError(
-                "n_best cannot be larger than n_components, but {} >  {}".format(
-                    self.n_best, self.n_components
-                )
+                f"n_best={self.n_best} must be <= n_components={self.n_components}."
             )
 
     def _fit(self, X):
