@@ -5,7 +5,11 @@ this module can be regarded as dimensionality reduction techniques.
 """
 
 
-from ._nmf import NMF, non_negative_factorization
+from ._nmf import (
+    NMF,
+    MiniBatchNMF,
+    non_negative_factorization,
+)
 from ._pca import PCA
 from ._incremental_pca import IncrementalPCA
 from ._kernel_pca import KernelPCA
@@ -31,6 +35,7 @@ __all__ = [
     "IncrementalPCA",
     "KernelPCA",
     "MiniBatchDictionaryLearning",
+    "MiniBatchNMF",
     "MiniBatchSparsePCA",
     "NMF",
     "PCA",
