@@ -34,12 +34,6 @@ def test_factor_analysis():
     # wlog, mean is 0
     X = np.dot(h, W) + noise
 
-    with pytest.raises(ValueError):
-        FactorAnalysis(svd_method="foo")
-    fa_fail = FactorAnalysis()
-    fa_fail.svd_method = "foo"
-    with pytest.raises(ValueError):
-        fa_fail.fit(X)
     fas = []
     for method in ["randomized", "lapack"]:
         fa = FactorAnalysis(n_components=n_components, svd_method=method)
@@ -101,9 +95,6 @@ def test_factor_analysis():
         assert not np.allclose(results[rot1], results[rot2])
         assert np.allclose(projections[rot1], projections[rot2], atol=3)
 
-    with pytest.raises(ValueError):
-        FactorAnalysis(rotation="not_implemented").fit_transform(X)
-
     # test against R's psych::principal with rotate="varimax"
     # (i.e., the values below stem from rotating the components in R)
     # R's factor analysis returns quite different values; therefore, we only
