@@ -15,13 +15,15 @@ from scipy import linalg
 from scipy.spatial.distance import pdist
 from scipy.spatial.distance import squareform
 from scipy.sparse import csr_matrix, issparse
+from numbers import Integral, Real
 from ..neighbors import NearestNeighbors
 from ..base import BaseEstimator
 from ..utils import check_random_state
 from ..utils._openmp_helpers import _openmp_effective_n_threads
 from ..utils.validation import check_non_negative
+from ..utils._param_validation import Interval, StrOptions, Hidden
 from ..decomposition import PCA
-from ..metrics.pairwise import pairwise_distances
+from ..metrics.pairwise import pairwise_distances, _VALID_METRICS
 
 # mypy error: Module 'sklearn.manifold' has no attribute '_utils'
 from . import _utils  # type: ignore
@@ -29,7 +31,6 @@ from . import _utils  # type: ignore
 # mypy error: Module 'sklearn.manifold' has no attribute '_barnes_hut_tsne'
 from . import _barnes_hut_tsne  # type: ignore
 
-
 MACHINE_EPSILON = np.finfo(np.double).eps
 
 
@@ -396,7 +397,6 @@ def _gradient_descent(
         kwargs["compute_error"] = check_convergence or i == n_iter - 1
 
         error, grad = objective(p, *args, **kwargs)
-        grad_norm = linalg.norm(grad)
 
         inc = update * grad < 0.0
         dec = np.invert(inc)
@@ -411,6 +411,7 @@ def _gradient_descent(
             toc = time()
             duration = toc - tic
             tic = toc
+            grad_norm = linalg.norm(grad)
 
             if verbose >= 2:
                 print(
@@ -443,7 +444,7 @@ def _gradient_descent(
 
 
 def trustworthiness(X, X_embedded, *, n_neighbors=5, metric="euclidean"):
-    r"""Expresses to what extent the local structure is retained.
+    r"""Indicate to what extent the local structure is retained.
 
     The trustworthiness is within [0, 1]. It is defined as
 
@@ -458,30 +459,29 @@ def trustworthiness(X, X_embedded, *, n_neighbors=5, metric="euclidean"):
     neighbors in the output space are penalised in proportion to their rank in
     the input space.
 
-    * "Neighborhood Preservation in Nonlinear Projection Methods: An
-      Experimental Study"
-      J. Venna, S. Kaski
-    * "Learning a Parametric Embedding by Preserving Local Structure"
-      L.J.P. van der Maaten
-
     Parameters
     ----------
-    X : ndarray of shape (n_samples, n_features) or (n_samples, n_samples)
+    X : {array-like, sparse matrix} of shape (n_samples, n_features) or \
+        (n_samples, n_samples)
         If the metric is 'precomputed' X must be a square distance
         matrix. Otherwise it contains a sample per row.
 
-    X_embedded : ndarray of shape (n_samples, n_components)
+    X_embedded : {array-like, sparse matrix} of shape (n_samples, n_components)
         Embedding of the training data in low-dimensional space.
 
     n_neighbors : int, default=5
-        Number of neighbors k that will be considered.
+        The number of neighbors that will be considered. Should be fewer than
+        `n_samples / 2` to ensure the trustworthiness to lies within [0, 1], as
+        mentioned in [1]_. An error will be raised otherwise.
 
     metric : str or callable, default='euclidean'
         Which metric to use for computing pairwise distances between samples
         from the original input space. If metric is 'precomputed', X must be a
-        matrix of pairwise distances or squared distances. Otherwise, see the
-        documentation of argument metric in sklearn.pairwise.pairwise_distances
-        for a list of available metrics.
+        matrix of pairwise distances or squared distances. Otherwise, for a list
+        of available metrics, see the documentation of argument metric in
+        `sklearn.pairwise.pairwise_distances` and metrics listed in
+        `sklearn.metrics.pairwise.PAIRWISE_DISTANCE_FUNCTIONS`. Note that the
+        "cosine" metric uses :func:`~sklearn.metrics.pairwise.cosine_distances`.
 
         .. versionadded:: 0.20
 
@@ -489,7 +489,24 @@ def trustworthiness(X, X_embedded, *, n_neighbors=5, metric="euclidean"):
     -------
     trustworthiness : float
         Trustworthiness of the low-dimensional embedding.
+
+    References
+    ----------
+    .. [1] Jarkko Venna and Samuel Kaski. 2001. Neighborhood
+           Preservation in Nonlinear Projection Methods: An Experimental Study.
+           In Proceedings of the International Conference on Artificial Neural Networks
+           (ICANN '01). Springer-Verlag, Berlin, Heidelberg, 485-491.
+
+    .. [2] Laurens van der Maaten. Learning a Parametric Embedding by Preserving
+           Local Structure. Proceedings of the Twelth International Conference on
+           Artificial Intelligence and Statistics, PMLR 5:384-391, 2009.
     """
+    n_samples = X.shape[0]
+    if n_neighbors >= n_samples / 2:
+        raise ValueError(
+            f"n_neighbors ({n_neighbors}) should be less than n_samples / 2"
+            f" ({n_samples / 2})"
+        )
     dist_X = pairwise_distances(X, metric=metric)
     if metric == "precomputed":
         dist_X = dist_X.copy()
@@ -507,7 +524,6 @@ def trustworthiness(X, X_embedded, *, n_neighbors=5, metric="euclidean"):
     # We build an inverted index of neighbors in the input space: For sample i,
     # we define `inverted_index[i]` as the inverted index of sorted distances:
     # inverted_index[i][ind_X[i]] = np.arange(1, n_sample + 1)
-    n_samples = X.shape[0]
     inverted_index = np.zeros((n_samples, n_samples), dtype=int)
     ordered_indices = np.arange(n_samples + 1)
     inverted_index[ordered_indices[:-1, np.newaxis], ind_X] = ordered_indices[1:]
@@ -550,7 +566,8 @@ class TSNE(BaseEstimator):
         is used in other manifold learning algorithms. Larger datasets
         usually require a larger perplexity. Consider selecting a value
         between 5 and 50. Different values can result in significantly
-        different results.
+        different results. The perplexity must be less that the number
+        of samples.
 
     early_exaggeration : float, default=12.0
         Controls how tight natural clusters in the original space are in
@@ -561,7 +578,7 @@ class TSNE(BaseEstimator):
         optimization, the early exaggeration factor or the learning rate
         might be too high.
 
-    learning_rate : float or 'auto', default=200.0
+    learning_rate : float or "auto", default="auto"
         The learning rate for t-SNE is usually in the range [10.0, 1000.0]. If
         the learning rate is too high, the data may look like a 'ball' with any
         point approximately equidistant from its nearest neighbours. If the
@@ -573,7 +590,10 @@ class TSNE(BaseEstimator):
         ours. So our learning_rate=200 corresponds to learning_rate=800 in
         those other implementations. The 'auto' option sets the learning_rate
         to `max(N / early_exaggeration / 4, 50)` where N is the sample size,
-        following [4] and [5]. This will become default in 1.2.
+        following [4] and [5].
+
+        .. versionchanged:: 1.2
+           The default value changed to `"auto"`.
 
     n_iter : int, default=1000
         Maximum number of iterations for the optimization. Should be at
@@ -604,13 +624,19 @@ class TSNE(BaseEstimator):
         the distance between them. The default is "euclidean" which is
         interpreted as squared euclidean distance.
 
-    init : {'random', 'pca'} or ndarray of shape (n_samples, n_components), \
-            default='random'
-        Initialization of embedding. Possible options are 'random', 'pca',
-        and a numpy array of shape (n_samples, n_components).
+    metric_params : dict, default=None
+        Additional keyword arguments for the metric function.
+
+        .. versionadded:: 1.1
+
+    init : {"random", "pca"} or ndarray of shape (n_samples, n_components), \
+            default="pca"
+        Initialization of embedding.
         PCA initialization cannot be used with precomputed distances and is
-        usually more globally stable than random initialization. `init='pca'`
-        will become default in 1.2.
+        usually more globally stable than random initialization.
+
+        .. versionchanged:: 1.2
+           The default value changed to `"pca"`.
 
     verbose : int, default=0
         Verbosity level.
@@ -621,7 +647,7 @@ class TSNE(BaseEstimator):
         initializations might result in different local minima of the cost
         function. See :term:`Glossary <random_state>`.
 
-    method : str, default='barnes_hut'
+    method : {'barnes_hut', 'exact'}, default='barnes_hut'
         By default the gradient calculation algorithm uses Barnes-Hut
         approximation running in O(NlogN) time. method='exact'
         will run on the slower, but exact, algorithm in O(N^2) time. The
@@ -652,18 +678,13 @@ class TSNE(BaseEstimator):
 
         .. versionadded:: 0.22
 
-    square_distances : True or 'legacy', default='legacy'
-        Whether TSNE should square the distance values. ``'legacy'`` means
-        that distance values are squared only when ``metric="euclidean"``.
-        ``True`` means that distance values are squared for all metrics.
+    square_distances : True, default='deprecated'
+        This parameter has no effect since distance values are always squared
+        since 1.1.
 
-        .. versionadded:: 0.24
-           Added to provide backward compatibility during deprecation of
-           legacy squaring behavior.
-        .. deprecated:: 0.24
-           Legacy squaring behavior was deprecated in 0.24. The ``'legacy'``
-           value will be removed in 1.1 (renaming of 0.26), at which point the
-           default value will change to ``True``.
+        .. deprecated:: 1.1
+             `square_distances` has no effect from 1.1 and will be removed in
+             1.3.
 
     Attributes
     ----------
@@ -684,6 +705,11 @@ class TSNE(BaseEstimator):
 
         .. versionadded:: 1.0
 
+    learning_rate_ : float
+        Effective learning rate.
+
+        .. versionadded:: 1.2
+
     n_iter_ : int
         Number of iterations run.
 
@@ -725,11 +751,36 @@ class TSNE(BaseEstimator):
     >>> from sklearn.manifold import TSNE
     >>> X = np.array([[0, 0, 0], [0, 1, 1], [1, 0, 1], [1, 1, 1]])
     >>> X_embedded = TSNE(n_components=2, learning_rate='auto',
-    ...                   init='random').fit_transform(X)
+    ...                   init='random', perplexity=3).fit_transform(X)
     >>> X_embedded.shape
     (4, 2)
     """
 
+    _parameter_constraints: dict = {
+        "n_components": [Interval(Integral, 1, None, closed="left")],
+        "perplexity": [Interval(Real, 0, None, closed="neither")],
+        "early_exaggeration": [Interval(Real, 1, None, closed="left")],
+        "learning_rate": [
+            StrOptions({"auto"}),
+            Interval(Real, 0, None, closed="neither"),
+        ],
+        "n_iter": [Interval(Integral, 250, None, closed="left")],
+        "n_iter_without_progress": [Interval(Integral, -1, None, closed="left")],
+        "min_grad_norm": [Interval(Real, 0, None, closed="left")],
+        "metric": [StrOptions(set(_VALID_METRICS) | {"precomputed"}), callable],
+        "metric_params": [dict, None],
+        "init": [
+            StrOptions({"pca", "random"}),
+            np.ndarray,
+        ],
+        "verbose": ["verbose"],
+        "random_state": ["random_state"],
+        "method": [StrOptions({"barnes_hut", "exact"})],
+        "angle": [Interval(Real, 0, 1, closed="both")],
+        "n_jobs": [None, Integral],
+        "square_distances": ["boolean", Hidden(StrOptions({"deprecated"}))],
+    }
+
     # Control the number of exploration iterations with early_exaggeration on
     _EXPLORATION_N_ITER = 250
 
@@ -742,18 +793,19 @@ class TSNE(BaseEstimator):
         *,
         perplexity=30.0,
         early_exaggeration=12.0,
-        learning_rate="warn",
+        learning_rate="auto",
         n_iter=1000,
         n_iter_without_progress=300,
         min_grad_norm=1e-7,
         metric="euclidean",
-        init="warn",
+        metric_params=None,
+        init="pca",
         verbose=0,
         random_state=None,
         method="barnes_hut",
         angle=0.5,
         n_jobs=None,
-        square_distances="legacy",
+        square_distances="deprecated",
     ):
         self.n_components = n_components
         self.perplexity = perplexity
@@ -763,69 +815,41 @@ class TSNE(BaseEstimator):
         self.n_iter_without_progress = n_iter_without_progress
         self.min_grad_norm = min_grad_norm
         self.metric = metric
+        self.metric_params = metric_params
         self.init = init
         self.verbose = verbose
         self.random_state = random_state
         self.method = method
         self.angle = angle
         self.n_jobs = n_jobs
-        # TODO Revisit deprecation of square_distances for 1.1-1.3 (#12401)
         self.square_distances = square_distances
 
+    def _check_params_vs_input(self, X):
+        if self.perplexity >= X.shape[0]:
+            raise ValueError("perplexity must be less than n_samples")
+
     def _fit(self, X, skip_num_points=0):
         """Private function to fit the model using X as training data."""
 
-        if isinstance(self.init, str) and self.init == "warn":
-            # See issue #18018
-            warnings.warn(
-                "The default initialization in TSNE will change "
-                "from 'random' to 'pca' in 1.2.",
-                FutureWarning,
-            )
-            self._init = "random"
-        else:
-            self._init = self.init
-        if self.learning_rate == "warn":
-            # See issue #18018
-            warnings.warn(
-                "The default learning rate in TSNE will change "
-                "from 200.0 to 'auto' in 1.2.",
-                FutureWarning,
-            )
-            self._learning_rate = 200.0
-        else:
-            self._learning_rate = self.learning_rate
-
-        if isinstance(self._init, str) and self._init == "pca" and issparse(X):
+        if isinstance(self.init, str) and self.init == "pca" and issparse(X):
             raise TypeError(
                 "PCA initialization is currently not supported "
                 "with the sparse input matrix. Use "
                 'init="random" instead.'
             )
-        if self.method not in ["barnes_hut", "exact"]:
-            raise ValueError("'method' must be 'barnes_hut' or 'exact'")
-        if self.angle < 0.0 or self.angle > 1.0:
-            raise ValueError("'angle' must be between 0.0 - 1.0")
-        if self.square_distances not in [True, "legacy"]:
-            raise ValueError("'square_distances' must be True or 'legacy'.")
-        if self._learning_rate == "auto":
-            # See issue #18018
-            self._learning_rate = X.shape[0] / self.early_exaggeration / 4
-            self._learning_rate = np.maximum(self._learning_rate, 50)
-        else:
-            if not (self._learning_rate > 0):
-                raise ValueError("'learning_rate' must be a positive number or 'auto'.")
-        if self.metric != "euclidean" and self.square_distances is not True:
+        if self.square_distances != "deprecated":
             warnings.warn(
-                "'square_distances' has been introduced in 0.24 to help phase "
-                "out legacy squaring behavior. The 'legacy' setting will be "
-                "removed in 1.1 (renaming of 0.26), and the default setting "
-                "will be changed to True. In 1.3, 'square_distances' will be "
-                "removed altogether, and distances will be squared by "
-                "default. Set 'square_distances'=True to silence this "
-                "warning.",
+                "The parameter `square_distances` has not effect and will be "
+                "removed in version 1.3.",
                 FutureWarning,
             )
+        if self.learning_rate == "auto":
+            # See issue #18018
+            self.learning_rate_ = X.shape[0] / self.early_exaggeration / 4
+            self.learning_rate_ = np.maximum(self.learning_rate_, 50)
+        else:
+            self.learning_rate_ = self.learning_rate
+
         if self.method == "barnes_hut":
             X = self._validate_data(
                 X,
@@ -838,7 +862,7 @@ class TSNE(BaseEstimator):
                 X, accept_sparse=["csr", "csc", "coo"], dtype=[np.float32, np.float64]
             )
         if self.metric == "precomputed":
-            if isinstance(self._init, str) and self._init == "pca":
+            if isinstance(self.init, str) and self.init == "pca":
                 raise ValueError(
                     'The parameter init="pca" cannot be used with metric="precomputed".'
                 )
@@ -866,16 +890,6 @@ class TSNE(BaseEstimator):
             )
         random_state = check_random_state(self.random_state)
 
-        if self.early_exaggeration < 1.0:
-            raise ValueError(
-                "early_exaggeration must be at least 1, but is {}".format(
-                    self.early_exaggeration
-                )
-            )
-
-        if self.n_iter < 250:
-            raise ValueError("n_iter should be at least 250")
-
         n_samples = X.shape[0]
 
         neighbors_nn = None
@@ -896,8 +910,9 @@ class TSNE(BaseEstimator):
                     # Also, Euclidean is slower for n_jobs>1, so don't set here
                     distances = pairwise_distances(X, metric=self.metric, squared=True)
                 else:
+                    metric_params_ = self.metric_params or {}
                     distances = pairwise_distances(
-                        X, metric=self.metric, n_jobs=self.n_jobs
+                        X, metric=self.metric, n_jobs=self.n_jobs, **metric_params_
                     )
 
             if np.any(distances < 0):
@@ -905,7 +920,7 @@ class TSNE(BaseEstimator):
                     "All distances should be positive, the metric given is not correct"
                 )
 
-            if self.metric != "euclidean" and self.square_distances is True:
+            if self.metric != "euclidean":
                 distances **= 2
 
             # compute the joint probability distribution for the input space
@@ -932,6 +947,7 @@ class TSNE(BaseEstimator):
                 n_jobs=self.n_jobs,
                 n_neighbors=n_neighbors,
                 metric=self.metric,
+                metric_params=self.metric_params,
             )
             t0 = time()
             knn.fit(X)
@@ -956,44 +972,34 @@ class TSNE(BaseEstimator):
             # Free the memory used by the ball_tree
             del knn
 
-            if self.square_distances is True or self.metric == "euclidean":
-                # knn return the euclidean distance but we need it squared
-                # to be consistent with the 'exact' method. Note that the
-                # the method was derived using the euclidean method as in the
-                # input space. Not sure of the implication of using a different
-                # metric.
-                distances_nn.data **= 2
+            # knn return the euclidean distance but we need it squared
+            # to be consistent with the 'exact' method. Note that the
+            # the method was derived using the euclidean method as in the
+            # input space. Not sure of the implication of using a different
+            # metric.
+            distances_nn.data **= 2
 
             # compute the joint probability distribution for the input space
             P = _joint_probabilities_nn(distances_nn, self.perplexity, self.verbose)
 
-        if isinstance(self._init, np.ndarray):
-            X_embedded = self._init
-        elif self._init == "pca":
+        if isinstance(self.init, np.ndarray):
+            X_embedded = self.init
+        elif self.init == "pca":
             pca = PCA(
                 n_components=self.n_components,
                 svd_solver="randomized",
                 random_state=random_state,
             )
             X_embedded = pca.fit_transform(X).astype(np.float32, copy=False)
-            # TODO: Update in 1.2
             # PCA is rescaled so that PC1 has standard deviation 1e-4 which is
             # the default value for random initialization. See issue #18018.
-            warnings.warn(
-                "The PCA initialization in TSNE will change to "
-                "have the standard deviation of PC1 equal to 1e-4 "
-                "in 1.2. This will ensure better convergence.",
-                FutureWarning,
-            )
-            # X_embedded = X_embedded / np.std(X_embedded[:, 0]) * 1e-4
-        elif self._init == "random":
+            X_embedded = X_embedded / np.std(X_embedded[:, 0]) * 1e-4
+        elif self.init == "random":
             # The embedding is initialized with iid samples from Gaussians with
             # standard deviation 1e-4.
-            X_embedded = 1e-4 * random_state.randn(n_samples, self.n_components).astype(
-                np.float32
-            )
-        else:
-            raise ValueError("'init' must be 'pca', 'random', or a numpy array")
+            X_embedded = 1e-4 * random_state.standard_normal(
+                size=(n_samples, self.n_components)
+            ).astype(np.float32)
 
         # Degrees of freedom of the Student's t-distribution. The suggestion
         # degrees_of_freedom = n_components - 1 comes from
@@ -1031,7 +1037,7 @@ class TSNE(BaseEstimator):
             "it": 0,
             "n_iter_check": self._N_ITER_CHECK,
             "min_grad_norm": self.min_grad_norm,
-            "learning_rate": self._learning_rate,
+            "learning_rate": self.learning_rate_,
             "verbose": self.verbose,
             "kwargs": dict(skip_num_points=skip_num_points),
             "args": [P, degrees_of_freedom, n_samples, self.n_components],
@@ -1090,7 +1096,8 @@ class TSNE(BaseEstimator):
 
         Parameters
         ----------
-        X : ndarray of shape (n_samples, n_features) or (n_samples, n_samples)
+        X : {array-like, sparse matrix} of shape (n_samples, n_features) or \
+            (n_samples, n_samples)
             If the metric is 'precomputed' X must be a square distance
             matrix. Otherwise it contains a sample per row. If the method
             is 'exact', X may be a sparse matrix of type 'csr', 'csc'
@@ -1105,6 +1112,8 @@ class TSNE(BaseEstimator):
         X_new : ndarray of shape (n_samples, n_components)
             Embedding of the training data in low-dimensional space.
         """
+        self._validate_params()
+        self._check_params_vs_input(X)
         embedding = self._fit(X)
         self.embedding_ = embedding
         return self.embedding_
@@ -1114,7 +1123,8 @@ class TSNE(BaseEstimator):
 
         Parameters
         ----------
-        X : ndarray of shape (n_samples, n_features) or (n_samples, n_samples)
+        X : {array-like, sparse matrix} of shape (n_samples, n_features) or \
+            (n_samples, n_samples)
             If the metric is 'precomputed' X must be a square distance
             matrix. Otherwise it contains a sample per row. If the method
             is 'exact', X may be a sparse matrix of type 'csr', 'csc'
@@ -1129,5 +1139,9 @@ class TSNE(BaseEstimator):
         X_new : array of shape (n_samples, n_components)
             Embedding of the training data in low-dimensional space.
         """
+        self._validate_params()
         self.fit_transform(X)
         return self
+
+    def _more_tags(self):
+        return {"pairwise": self.metric == "precomputed"}
