@@ -30,6 +30,8 @@ from sklearn.metrics import cohen_kappa_score
 from sklearn.metrics import confusion_matrix
 from sklearn.metrics import coverage_error
 from sklearn.metrics import d2_tweedie_score
+from sklearn.metrics import d2_pinball_score
+from sklearn.metrics import d2_absolute_error_score
 from sklearn.metrics import det_curve
 from sklearn.metrics import explained_variance_score
 from sklearn.metrics import f1_score
@@ -112,6 +114,8 @@ REGRESSION_METRICS = {
     "mean_gamma_deviance": mean_gamma_deviance,
     "mean_compound_poisson_deviance": partial(mean_tweedie_deviance, power=1.4),
     "d2_tweedie_score": partial(d2_tweedie_score, power=1.4),
+    "d2_pinball_score": d2_pinball_score,
+    "d2_absolute_error_score": d2_absolute_error_score,
 }
 
 CLASSIFICATION_METRICS = {
@@ -446,6 +450,8 @@ MULTIOUTPUT_METRICS = {
     "explained_variance_score",
     "mean_absolute_percentage_error",
     "mean_pinball_loss",
+    "d2_pinball_score",
+    "d2_absolute_error_score",
 }
 
 # Symmetric with respect to their input arguments y_true and y_pred
@@ -507,12 +513,13 @@ NOT_SYMMETRIC_METRICS = {
     "macro_f2_score",
     "macro_precision_score",
     "macro_recall_score",
-    "log_loss",
     "hinge_loss",
     "mean_gamma_deviance",
     "mean_poisson_deviance",
     "mean_compound_poisson_deviance",
     "d2_tweedie_score",
+    "d2_pinball_score",
+    "d2_absolute_error_score",
     "mean_absolute_percentage_error",
 }
 
@@ -902,7 +909,11 @@ invalids_nan_inf = [
 )
 @pytest.mark.parametrize("y_true, y_score", invalids_nan_inf)
 def test_regression_thresholded_inf_nan_input(metric, y_true, y_score):
-    with pytest.raises(ValueError, match="contains NaN, infinity"):
+    # Reshape since coverage_error only accepts 2D arrays.
+    if metric == coverage_error:
+        y_true = [y_true]
+        y_score = [y_score]
+    with pytest.raises(ValueError, match=r"contains (NaN|infinity)"):
         metric(y_true, y_score)
 
 
@@ -913,12 +924,29 @@ def test_regression_thresholded_inf_nan_input(metric, y_true, y_score):
     # Add an additional case for classification only
     # non-regression test for:
     # https://github.com/scikit-learn/scikit-learn/issues/6809
-    [([np.nan, 1, 2], [1, 2, 3])],  # type: ignore
+    [
+        ([np.nan, 1, 2], [1, 2, 3]),
+        ([np.inf, 1, 2], [1, 2, 3]),
+    ],  # type: ignore
 )
 def test_classification_inf_nan_input(metric, y_true, y_score):
     """check that classification metrics raise a message mentioning the
     occurrence of non-finite values in the target vectors."""
-    err_msg = "Input contains NaN, infinity or a value too large"
+    if not np.isfinite(y_true).all():
+        input_name = "y_true"
+        if np.isnan(y_true).any():
+            unexpected_value = "NaN"
+        else:
+            unexpected_value = "infinity or a value too large"
+    else:
+        input_name = "y_pred"
+        if np.isnan(y_score).any():
+            unexpected_value = "NaN"
+        else:
+            unexpected_value = "infinity or a value too large"
+
+    err_msg = f"Input {input_name} contains {unexpected_value}"
+
     with pytest.raises(ValueError, match=err_msg):
         metric(y_true, y_score)
 
