@@ -2,6 +2,7 @@
 #         Thierry Guillemot <thierry.guillemot.work@gmail.com>
 # License: BSD 3 clause
 
+import itertools
 import re
 import sys
 import copy
@@ -127,66 +128,6 @@ def test_gaussian_mixture_attributes():
     rng = np.random.RandomState(0)
     X = rng.rand(10, 2)
 
-    n_components_bad = 0
-    gmm = GaussianMixture(n_components=n_components_bad)
-    msg = (
-        f"Invalid value for 'n_components': {n_components_bad} "
-        "Estimation requires at least one component"
-    )
-    with pytest.raises(ValueError, match=msg):
-        gmm.fit(X)
-
-    # covariance_type should be in [spherical, diag, tied, full]
-    covariance_type_bad = "bad_covariance_type"
-    gmm = GaussianMixture(covariance_type=covariance_type_bad)
-    msg = (
-        f"Invalid value for 'covariance_type': {covariance_type_bad} "
-        "'covariance_type' should be in ['spherical', 'tied', 'diag', 'full']"
-    )
-    with pytest.raises(ValueError):
-        gmm.fit(X)
-
-    tol_bad = -1
-    gmm = GaussianMixture(tol=tol_bad)
-    msg = (
-        f"Invalid value for 'tol': {tol_bad:.5f} "
-        "Tolerance used by the EM must be non-negative"
-    )
-    with pytest.raises(ValueError, match=msg):
-        gmm.fit(X)
-
-    reg_covar_bad = -1
-    gmm = GaussianMixture(reg_covar=reg_covar_bad)
-    msg = (
-        f"Invalid value for 'reg_covar': {reg_covar_bad:.5f} "
-        "regularization on covariance must be non-negative"
-    )
-    with pytest.raises(ValueError, match=msg):
-        gmm.fit(X)
-
-    max_iter_bad = 0
-    gmm = GaussianMixture(max_iter=max_iter_bad)
-    msg = (
-        f"Invalid value for 'max_iter': {max_iter_bad} "
-        "Estimation requires at least one iteration"
-    )
-    with pytest.raises(ValueError, match=msg):
-        gmm.fit(X)
-
-    n_init_bad = 0
-    gmm = GaussianMixture(n_init=n_init_bad)
-    msg = (
-        f"Invalid value for 'n_init': {n_init_bad} Estimation requires at least one run"
-    )
-    with pytest.raises(ValueError, match=msg):
-        gmm.fit(X)
-
-    init_params_bad = "bad_method"
-    gmm = GaussianMixture(init_params=init_params_bad)
-    msg = f"Unimplemented initialization method '{init_params_bad}'"
-    with pytest.raises(ValueError, match=msg):
-        gmm.fit(X)
-
     # test good parameters
     n_components, tol, n_init, max_iter, reg_covar = 2, 1e-4, 3, 30, 1e-1
     covariance_type, init_params = "full", "random"
@@ -423,7 +364,7 @@ def test_suffstat_sk_diag():
     covars_pred_diag = _estimate_gaussian_covariances_diag(resp, X, nk, xk, 0)
 
     ecov = EmpiricalCovariance()
-    for (cov_full, cov_diag) in zip(covars_pred_full, covars_pred_diag):
+    for cov_full, cov_diag in zip(covars_pred_full, covars_pred_diag):
         ecov.covariance_ = np.diag(np.diag(cov_full))
         cov_diag = np.diag(cov_diag)
         assert_almost_equal(ecov.error_norm(cov_diag, norm="frobenius"), 0)
@@ -431,7 +372,7 @@ def test_suffstat_sk_diag():
 
     # check the precision computation
     precs_chol_pred = _compute_precision_cholesky(covars_pred_diag, "diag")
-    assert_almost_equal(covars_pred_diag, 1.0 / precs_chol_pred ** 2)
+    assert_almost_equal(covars_pred_diag, 1.0 / precs_chol_pred**2)
 
 
 def test_gaussian_suffstat_sk_spherical():
@@ -453,7 +394,7 @@ def test_gaussian_suffstat_sk_spherical():
 
     # check the precision computation
     precs_chol_pred = _compute_precision_cholesky(covars_pred_spherical, "spherical")
-    assert_almost_equal(covars_pred_spherical, 1.0 / precs_chol_pred ** 2)
+    assert_almost_equal(covars_pred_spherical, 1.0 / precs_chol_pred**2)
 
 
 def test_compute_log_det_cholesky():
@@ -470,7 +411,7 @@ def test_compute_log_det_cholesky():
         elif covar_type == "diag":
             predected_det = np.array([np.prod(cov) for cov in covariance])
         elif covar_type == "spherical":
-            predected_det = covariance ** n_features
+            predected_det = covariance**n_features
 
         # We compute the cholesky decomposition of the covariance matrix
         expected_det = _compute_log_det_cholesky(
@@ -1263,6 +1204,67 @@ def test_gaussian_mixture_setting_best_params():
         assert hasattr(gmm, attr)
 
 
+@pytest.mark.parametrize(
+    "init_params", ["random", "random_from_data", "k-means++", "kmeans"]
+)
+def test_init_means_not_duplicated(init_params, global_random_seed):
+    # Check that all initialisations provide not duplicated starting means
+    rng = np.random.RandomState(global_random_seed)
+    rand_data = RandomData(rng, scale=5)
+    n_components = rand_data.n_components
+    X = rand_data.X["full"]
+
+    gmm = GaussianMixture(
+        n_components=n_components, init_params=init_params, random_state=rng, max_iter=0
+    )
+    gmm.fit(X)
+
+    means = gmm.means_
+    for i_mean, j_mean in itertools.combinations(means, r=2):
+        assert not np.allclose(i_mean, j_mean)
+
+
+@pytest.mark.parametrize(
+    "init_params", ["random", "random_from_data", "k-means++", "kmeans"]
+)
+def test_means_for_all_inits(init_params, global_random_seed):
+    # Check fitted means properties for all initializations
+    rng = np.random.RandomState(global_random_seed)
+    rand_data = RandomData(rng, scale=5)
+    n_components = rand_data.n_components
+    X = rand_data.X["full"]
+
+    gmm = GaussianMixture(
+        n_components=n_components, init_params=init_params, random_state=rng
+    )
+    gmm.fit(X)
+
+    assert gmm.means_.shape == (n_components, X.shape[1])
+    assert np.all(X.min(axis=0) <= gmm.means_)
+    assert np.all(gmm.means_ <= X.max(axis=0))
+    assert gmm.converged_
+
+
+def test_max_iter_zero():
+    # Check that max_iter=0 returns initialisation as expected
+    # Pick arbitrary initial means and check equal to max_iter=0
+    rng = np.random.RandomState(0)
+    rand_data = RandomData(rng, scale=5)
+    n_components = rand_data.n_components
+    X = rand_data.X["full"]
+    means_init = [[20, 30], [30, 25]]
+    gmm = GaussianMixture(
+        n_components=n_components,
+        random_state=rng,
+        means_init=means_init,
+        tol=1e-06,
+        max_iter=0,
+    )
+    gmm.fit(X)
+
+    assert_allclose(gmm.means_, means_init)
+
+
 def test_gaussian_mixture_precisions_init_diag():
     """Check that we properly initialize `precision_cholesky_` when we manually
     provide the precision matrix.
@@ -1322,3 +1324,14 @@ def test_gaussian_mixture_precisions_init_diag():
     assert_allclose(
         gm_with_init.precisions_cholesky_, gm_without_init.precisions_cholesky_
     )
+
+
+def test_gaussian_mixture_single_component_stable():
+    """
+    Non-regression test for #23032 ensuring 1-component GM works on only a
+    few samples.
+    """
+    rng = np.random.RandomState(0)
+    X = rng.multivariate_normal(np.zeros(2), np.identity(2), size=3)
+    gm = GaussianMixture(n_components=1)
+    gm.fit(X).sample()
