@@ -231,17 +231,6 @@ def test_classifier_prediction_independent_of_X(strategy):
     assert_array_equal(predictions1, predictions2)
 
 
-def test_classifier_exceptions():
-    clf = DummyClassifier(strategy="unknown")
-    with pytest.raises(ValueError):
-        clf.fit([], [])
-
-    with pytest.raises(NotFittedError):
-        clf.predict([])
-    with pytest.raises(NotFittedError):
-        clf.predict_proba([])
-
-
 def test_mean_strategy_regressor():
 
     random_state = np.random.RandomState(seed=1)
@@ -379,28 +368,11 @@ def test_quantile_invalid():
     X = [[0]] * 5  # ignored
     y = [0] * 5  # ignored
 
-    est = DummyRegressor(strategy="quantile")
-    with pytest.raises(ValueError):
-        est.fit(X, y)
-
     est = DummyRegressor(strategy="quantile", quantile=None)
-    with pytest.raises(ValueError):
-        est.fit(X, y)
-
-    est = DummyRegressor(strategy="quantile", quantile=[0])
-    with pytest.raises(ValueError):
-        est.fit(X, y)
-
-    est = DummyRegressor(strategy="quantile", quantile=-0.1)
-    with pytest.raises(ValueError):
-        est.fit(X, y)
-
-    est = DummyRegressor(strategy="quantile", quantile=1.1)
-    with pytest.raises(ValueError):
-        est.fit(X, y)
-
-    est = DummyRegressor(strategy="quantile", quantile="abc")
-    with pytest.raises(TypeError):
+    err_msg = (
+        "When using `strategy='quantile', you have to specify the desired quantile"
+    )
+    with pytest.raises(ValueError, match=err_msg):
         est.fit(X, y)
 
 
@@ -425,6 +397,9 @@ def test_constant_strategy_regressor():
     reg.fit(X, y)
     assert_array_equal(reg.predict(X), [43] * len(X))
 
+    # non-regression test for #22478
+    assert not isinstance(reg.constant, np.ndarray)
+
 
 def test_constant_strategy_multioutput_regressor():
 
@@ -459,21 +434,13 @@ def test_y_mean_attribute_regressor():
     assert est.constant_ == np.mean(y)
 
 
-def test_unknown_strategey_regressor():
-    X = [[0]] * 5
-    y = [1, 2, 4, 6, 8]
-
-    est = DummyRegressor(strategy="gona")
-    with pytest.raises(ValueError):
-        est.fit(X, y)
-
-
 def test_constants_not_specified_regressor():
     X = [[0]] * 5
     y = [1, 2, 4, 6, 8]
 
     est = DummyRegressor(strategy="constant")
-    with pytest.raises(TypeError):
+    err_msg = "Constant target value has to be specified"
+    with pytest.raises(TypeError, match=err_msg):
         est.fit(X, y)
 
 
@@ -483,7 +450,8 @@ def test_constant_size_multioutput_regressor():
     y = random_state.randn(10, 5)
 
     est = DummyRegressor(strategy="constant", constant=[1, 2, 3, 4])
-    with pytest.raises(ValueError):
+    err_msg = r"Constant target value should have shape \(5, 1\)."
+    with pytest.raises(ValueError, match=err_msg):
         est.fit(X, y)
 
 
@@ -551,7 +519,6 @@ def test_constant_strategy_exceptions(y, params, err_msg):
     X = [[0], [0], [0], [0]]
 
     clf = DummyClassifier(strategy="constant", **params)
-
     with pytest.raises(ValueError, match=err_msg):
         clf.fit(X, y)
 
@@ -724,18 +691,3 @@ def test_dtype_of_classifier_probas(strategy):
     probas = model.fit(X, y).predict_proba(X)
 
     assert probas.dtype == np.float64
-
-
-# TODO: remove in 1.2
-@pytest.mark.filterwarnings("ignore:`n_features_in_` is deprecated")
-@pytest.mark.parametrize("Dummy", (DummyRegressor, DummyClassifier))
-def test_n_features_in_(Dummy):
-    X = [[1, 2]]
-    y = [0]
-    d = Dummy()
-    assert not hasattr(d, "n_features_in_")
-    d.fit(X, y)
-
-    with pytest.warns(FutureWarning, match="`n_features_in_` is deprecated"):
-        n_features_in = d.n_features_in_
-    assert n_features_in is None
